#!/bin/sh
# Script to install semigroupe
# Usage : you need to login as root and then run the command
#
# ./install_semigroupe.sh homedirectory username
#
# where username is you home directory (something like /home/yourname). If
# you just type
#
# sudo ./install_semigroupe.sh
#
# it will be installed in the root directory (not recommended)
# the command semigroupe will be intalled in /usr/local/bin 
# the directory BABEL will be created in /usr/local/etc
#
# sudo ./install_semigroupe.sh "/usr/bin" "/BABELpath/BABELfolder"
#
# tar -zxvf semigroupe.tar.gz


if [ $# -gt 0 ]
then  
  INSTALL_HOME="$1"
else
  INSTALL_HOME="$HOME"
fi

if [ $# -gt 1 ]
then  
  USER_NAME="$2"
else
  USER_NAME="root"
fi

INSTALL_LOCAL="/usr/local"
INSTALL_DIR="/usr/local/bin"
INSTALL_BABEL="/usr/local/etc"
INSTALL_DOCUMENTS="$INSTALL_HOME/Documents"
INSTALL_SEMIGROUPE="$INSTALL_DOCUMENTS/Semigroupe"
INSTALL_EXAMPLES="$INSTALL_SEMIGROUPE/Examples"
INSTALL_LATEX="$INSTALL_SEMIGROUPE/LaTeXFiles"

# Creating directories if needed
if (test -d $INSTALL_LOCAL) then
	echo "Directory $INSTALL_LOCAL found"
else
	echo "No directory $INSTALL_LOCAL ! Creating a new one";
	mkdir $INSTALL_LOCAL;
fi

if (test -d $INSTALL_BABEL) then
	echo "Directory $INSTALL_BABEL found"
else
	echo "No directory $INSTALL_BABEL ! Creating a new one";
	mkdir $INSTALL_BABEL;
fi

if (test -d $INSTALL_DIR) then
	echo "Directory $INSTALL_DIR found"
else
	echo "No directory $INSTALL_DIR ! Creating a new one";
	mkdir $INSTALL_DIR;
fi

if (test -d $INSTALL_DOCUMENTS) then
	echo "Directory $INSTALL_DOCUMENTS found"
else
	echo "No directory $INSTALL_DOCUMENTS ! Creating a new one";
	mkdir $INSTALL_DOCUMENTS;
fi

if (test -d $INSTALL_SEMIGROUPE) then
	echo "Directory $INSTALL_SEMIGROUPE found"
else
	echo "No directory $INSTALL_SEMIGROUPE ! Creating a new one";
	mkdir $INSTALL_SEMIGROUPE;
fi

if (test -d $INSTALL_EXAMPLES) then
	echo "Directory $INSTALL_EXAMPLES found"
else
	echo "No directory $INSTALL_EXAMPLES ! Creating a new one";
	mkdir $INSTALL_EXAMPLES;
fi

if (test -d $INSTALL_LATEX) then
	echo "Directory $INSTALL_LATEX found"
else
	echo "No directory $INSTALL_LATEX ! Creating a new one";
	mkdir $INSTALL_LATEX;
fi

# Copying files
echo "Installing .Semigroupe2Prefs in your home directory"
cp Semigroupe2Prefs $INSTALL_HOME/.Semigroupe2Prefs
echo "Installing the LaTeX macros"
cp MacrosJEP.tex $INSTALL_LATEX
echo "Installing the Babel folder in /usr/local/etc"
cp -R Babel $INSTALL_BABEL

cd Sources
# If no argument, default installation
if [ $# -gt 0 ]
then  
  sed s!/usr/local/bin!$INSTALL_DIR! Preferences.h > Preferences.temp;
  mv Preferences.temp Preferences.h;
fi

# Compiling
echo Compiling...
make semigroupe2
mv semigroupe2.out ..
make clean

cd ..
if (test -e "semigroupe2") then
	echo removing old file "semigroupe2"
	rm semigroupe2
fi

# Creating the command semigroupe
echo "#! /bin/sh" > semigroupe2
echo semigroupe2.out "$INSTALL_HOME" "$INSTALL_BABEL" >> semigroupe2
chmod +x semigroupe2

mv semigroupe2 $INSTALL_DIR
mv semigroupe2.out $INSTALL_DIR

# Changing the files owners
echo "Change of file owners"
chown -R $USER_NAME $INSTALL_SEMIGROUPE
chown -R $USER_NAME $INSTALL_EXAMPLES
chown -R $USER_NAME $INSTALL_LATEX
chown $USER_NAME $INSTALL_HOME/.Semigroupe2Prefs

# Cleaning
cd $INSTALL_DOCUMENTS
rm semigroupe.tar.gz
yes | rm -r SemigroupeFolder
echo "Installation ready"
